%Per pulire il workspace
clear variables
%Per pulire la command window
clc

%%%%%
% --------------------------------------------------------------------- %
%              Rishio e Rendimento di Portafoglio           
% --------------------------------------------------------------------- %

% Come prima cosa carichiamo i dati:

% Relativi alla serie dei prezzi dell'indice SPI 500
% Possiamo caricarli come tabella, in questo modo possiamo caricare anche 
% le date e i nomi dei campi
s_t= readtable('Dataset.xlsx','Sheet','Stock Prices');

%Visualizziamo la serie dei prezzi tramite la funzione plot(X,Y)
figure(1)
%plot( D.Date_obs,P(:,[1 40]))
plot(s_t,"Date_obs",[2 41])
legend
title('Stocks 1 and 40 - Prices');

% Oppure possiamo caricare i dati come matrice
P  = readmatrix('Dataset.xlsx','Sheet','Stock Prices','Range','B2:SJ3274');
% Possiamo comunque caricare seperatamente le Date e i Nomi
Names = readcell('Dataset.xlsx','Sheet','Stock Prices','Range','B1:SJ1');
D  = s_t.Date_obs;

%Dato il vettore dei prezzi giornalieri è possibile calcolare il rendimento
%netto in punti percentuali come segue: 
%Rn = (P_t - P_(t-1))/P_(t-1)*100
%%%%%%%%% Compute the Net Returns

R = 100*((P(2:end,:)-P(1:end-1,:))./P(1:end-1,:));

%I rendimenti logaritmici sono invece calcolati con la seguente formula:
%Rl=log(P_t/P_(t-1))*100

%creiamo le date rispetto al vettore dei rendimenti
D_R = D(2:end,:);

% selezioniamo i dati relativi al 1 e al 40 asset
R1  = R(:,1);
R40 = R(:,40);
Namesp = Names([1 40]);

% Grafico dei rendimenti delle due serie selezionate
figure(3)
plot(D_R,R1);
title('Stocks 1 and 40 - Returns');
ylabel('R');
xlabel('Date');
hold on
plot(D_R,R40,'r');
legend('R_{1}','R_{40}');
hold off

%Creiamo un portafoglio composto dai titoli 1 e 40 con pesi w1 = 70% e w2 = 30%
w1 = 0.7;
w2 = 1-w1;

%Calcoliamo la media e della deviazione standard del portafoglio
%Possiamo farlo in 3 modi
% 1. Calcoliamo prima la media e la dev std dei titoli e poi ne calcoliamo
%media e varianza

mu_1 = w1*mean(R1)+w2*mean(R40);
%mu = [w1 w2]*mean(Rp)';
C  = cov(R1,R40);  
Vp_1 = (w1^2)*var(R1)+(w2^2)*var(R40)+2*w1*w2*C(2,1);
Sp_1 = sqrt(Vp_1);

% 2. Cacoliamo prima la serie dei rendimenti storici del portafoglio e poi
% %ne calcoliamo media e varianza

Rp_2 = w1*R1 + w2*R40;
mu_2 = mean(Rp_2);
Sp_2 = std(Rp_2);

%3. Utilizziamo il calcolo matriciale
w =  [w1; w2];
mu_3 = [mean(R1) mean(R40)]*w;
Sp_3 = sqrt(w'*C*w);

%Calcoliamo il rendimento di un portafoglio equipesato con i primi 10 asset
% selecting 10 stocks
Rp_t = R(:,1:10);

% in questo modo calcoliamo la media per riga escludendo le osservazioni
% mancanti

R_10 = mean(Rp_t,2,'omitnan'); 

%calcoliamo poi la media per colonna
Rp_4 = mean(R_10);
Sp_4 = std(R_10);

% un modo diverso per affrontare le osservazioni mancanti
% calcoliamo prima la media storica per singolo asset escludendo i nan
R_10_2 = mean(Rp_t,'omitnan');
%%%%
%poi calcoliamo la media e la varianza di portafoglio
Rp_5 = mean(R_10_2,'omitnan');
Sp_5 = std(R_10_2,'omitnan');

%%%%%
% --------------------------------------------------------------------- %
%              Introduzione alla frontiera Efficiente           
% --------------------------------------------------------------------- %

% creiamo un portafoglio con due stock
Stocks = [494 268];
Port= R(:,Stocks);

% Calcoliamo il rendimento atteso e il rischio di ogni stock
ExpRet  = mean(Port,1);
Risk = std(Port,1);

% Creiamo il vettore dei pesi
w1 = 0.3;
w2 = 0.7;
w = [w1 w2]';

%calcoliamo il rendimento atteso e la deviazione standard di portafoglio 
%con notazione matriciale
Rp = ExpRet*w;
C  = cov(Port); 
Vp = w'*C*w;
Sp = sqrt(Vp);

% Facciamo il grafico dei due stock sul piano rischio rendimento
% Plot of the two stocks on the risk-return plane
 figure(2)
 plot(Sp,Rp,'bx','MarkerSize',7)
 hold on
 plot(std(Port),mean(Port),'ro','MarkerSize',7)
 ylim([0 0.3])
 xlim([0 5])
 xlabel('Risk')
 ylabel('Returns')
 legend('Portfolio','Assets')
 hold off

 
 % per costruire la Frontiera ripetiamo questo grafico per diverse
 % composizioni di portafoglio
 

w1 = 0.01:0.02:0.99;
w2 = 1-w1;
w = [w1' w2'];


Rp2 = ExpRet*w';
Vp2 = diag(w*C*w');
Sp2 = sqrt(Vp);
figure(3)
plot(Sp,Rp,'bx')
ylim([0 0.3])
xlim([0 5])

for i= 0.01:0.02:0.99 
    w1_tmp = i;
    w2_tmp = 1-i;    
    
    w_tmp = [w1_tmp w2_tmp]';
    Rp_tmp = ExpRet*w_tmp;
    Vp_tmp = w_tmp'*C*w_tmp;
    Sp_tmp = sqrt(Vp_tmp);
    
    figure(2)
    L(1) = plot(Sp_tmp,Rp_tmp,'bx','MarkerSize',7);
    hold on
    ylim([0 0.3])
    xlim([0 5])
    
end

% plot the chosen assets correpsonding to portfolios w1 = 0 and w2 = 0
L(2) = plot(Risk,ExpRet,'ro','MarkerSize',7);

%plot portfolio with w1 = 0.7 and w2 = 0.3
L(3) = plot(Sp,Rp,'r.','MarkerSize',20);

xlabel('\sigma')
ylabel('R')
title('Efficient Frontier')
%nella legenda richiamiamo la variabile L per visualizzare la legenda
%corretta
legend(L,'Available portfolios','Single Stocks','Chosen portfoglio w_1 = 70%, w_2 = 30%')
hold off

%Ora utilizziamo la funzione per fare il grafico delle frontiera con
%diverse correlazioni
corr_tit = corr(Port);

corr_dy =[-1 1 corr_tit(2,1)]; 

color= ['r.';'b.';'g.']; 

for j = 1:3
    L(j) = EfficientFrontier(Port,corr_dy(j),3,color(j,:));
    hold on
end 

L(j+1) = plot(Risk,ExpRet,'m.','MarkerSize',20);
L(j+2) = plot(Sp,Rp,'b.','MarkerSize',20);
xlabel('\sigma')
ylabel('R')
title('Efficient Frontier')
legend(L,'\rho = -1','\rho = 1',['\rho =' num2str(corr_dy(3))] ,'Single Stocks','Portfolio w_1 = 30%, w_2 = 70%','Location','northwest')
hold off
